#include "finder.h"
#include "resource.h"
#include <cstdio>

#define IDT_TIMER1	100

HWND found_hwnd = NULL;
HPEN g_hRectanglePen;

long RefreshWindow (HWND hwndWindowToBeRefreshed)
{
	long lRet = 0;
	InvalidateRect (hwndWindowToBeRefreshed, NULL, TRUE);
	UpdateWindow (hwndWindowToBeRefreshed);
	RedrawWindow (hwndWindowToBeRefreshed, NULL, NULL, RDW_FRAME | RDW_INVALIDATE | RDW_UPDATENOW | RDW_ALLCHILDREN);
	return lRet;
}

long HighlightFoundWindow(HWND hwndFoundWindow)
{
	HDC		hWindowDC = NULL;  // The DC of the found window.
	HGDIOBJ	hPrevPen = NULL;   // Handle of the existing pen in the DC of the found window.
	HGDIOBJ	hPrevBrush = NULL; // Handle of the existing brush in the DC of the found window.
	RECT		rect;              // Rectangle area of the found window.
	long		lRet = 0;
	GetWindowRect (hwndFoundWindow, &rect);
	hWindowDC = GetWindowDC (hwndFoundWindow);
	if (hWindowDC)
	{
		hPrevPen = SelectObject (hWindowDC, g_hRectanglePen);
		hPrevBrush = SelectObject (hWindowDC, GetStockObject(HOLLOW_BRUSH));
		Rectangle (hWindowDC, 0, 0, rect.right - rect.left, rect.bottom - rect.top);
		SelectObject (hWindowDC, hPrevPen);
		SelectObject (hWindowDC, hPrevBrush);
		ReleaseDC (hwndFoundWindow, hWindowDC);
	}
	return lRet;
}

void DisplayWindowInfo(HWND dialog_hwnd, HWND found_hwnd)
{
	char temp[256], class_name[256];
	GetClassNameA(found_hwnd, class_name, sizeof (class_name) - 1);
	sprintf(temp, "HWND: %08X", found_hwnd);
	SetDlgItemTextA(dialog_hwnd, IDC_HWND, temp);
	sprintf(temp, "Class: %s", class_name);
	SetDlgItemTextA(dialog_hwnd, IDC_CLASS, temp);
}

BOOL CALLBACK FinderDlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam)
{
    switch(Message)
    {
        case WM_INITDIALOG:
			SetWindowPos(hwnd, HWND_TOP, 10, 10, 0, 0, SWP_NOSIZE);
			SetTimer(hwnd, IDT_TIMER1, 50, NULL);
			return TRUE;

        case WM_COMMAND:
            switch(LOWORD(wParam))
            {
				case IDCANCEL:
					found_hwnd = 0;
                case IDOK:
					KillTimer(hwnd, IDT_TIMER1);
					if ( found_hwnd != NULL )
						RefreshWindow(found_hwnd);
                    EndDialog(hwnd, LOWORD(wParam));
                break;
            }
        break;

		case WM_TIMER:
			switch(wParam)
			{
				case IDT_TIMER1:
					POINT p;
					GetCursorPos(&p);
					HWND new_hwnd = WindowFromPoint(p);
					if ( new_hwnd != found_hwnd )
					{
						if ( found_hwnd != NULL )
							RefreshWindow(found_hwnd);
						found_hwnd = new_hwnd;
						DisplayWindowInfo(hwnd, found_hwnd);
						HighlightFoundWindow(found_hwnd);
					}
					break;
			}
		break;

        default:
            return FALSE;
    }
    return TRUE;
}

HWND Finder_FindHWND()
{
	g_hRectanglePen = CreatePen(PS_SOLID, 3, RGB(256, 0, 0));
	DialogBox(GetModuleHandle(NULL), MAKEINTRESOURCE(IDD_FINDER), NULL, FinderDlgProc);
	return found_hwnd;
}